// Bugs-B-Gone for C++
//
// Version 1.3 (Sunday, 20 June 1999)
//
// Created by Matthijs Hollemans 
// matthijs@weirdexperiments.com
// http://www.weirdexperiments.com/
//
// Bugs-B-Gone is public domain. No warranties expressed or implied.
//
// This package offers a few simple macros that allow you to write debugging 
// output to stdout. Nothing special, but it works for me. For more "internal
// debugging" power, you may wish to look at Fred Fish's DBUG package.
//
// Typically, you would put something like this at the top of your main 
// source (or include) file:
// 
//   #define  BUGS_B_GONE
//   #define  BBG_PREFIX "MyApp"
//   #include "Bugs-B-Gone.h"
//
// The various symbols have the following meaning:
//
//   BUGS_B_GONE          -- Enables or disables the Bugs-B-Gone package.
//
//   BBG_PREFIX           -- Defines the text that is prepended to all
//                           debugging output, typically the name of your
//                           application. If you don't define BBG_PREFIX 
//                           yourself, it defaults to "Bugs-B-Gone".
//
// Meet your slaves, oh master:
//
//   BBG_WRITE((Args))    -- Outputs the message given by Args. The argument
//                           takes the variable argument form of a printf()
//                           call and must be wrapped inside a second set of
//                           parenthesis; for example:
//                           BBG_WRITE(("Hi %s", "there!"))
//
//   BBG_ASSERT(Args)     -- Outputs the name and line of the current source
//                           code file if the condition given by Args (which
//                           can be any normal C or C++ expression) evaluates 
//                           to false, but doesn't halt execution of the
//                           program.
//
//   BBG_CHECKPOINT       -- Outputs the name and line of the source code 
//                           file that contains this checkpoint.
//
//   BBG_ENTER(FuncName)  -- Outputs a string that indicates that the code 
//                           has just entered a particular function.
//
//   BBG_LEAVE(FuncName)  -- Outputs a string that indicates that the code 
//                           is about to leave a particular function.
//
//   BBG_EXEC(Statements) -- Executes a section of code given by Statements,
//                           for example: DEBUG_EXEC(return -1)
//
// You don't need to separate these macros from other statements with a
// semicolon.

#ifndef BUGS_B_GONE_H
#define BUGS_B_GONE_H

#ifndef BBG_PREFIX
#define BBG_PREFIX "Bugs-B-Gone"
#endif

#ifdef BUGS_B_GONE

#include <stdio.h>	// printf

#define BBG_WRITE(Args) {printf("%s ",BBG_PREFIX); printf Args; printf("\n");}
#define BBG_ASSERT(Args) {if (!(Args)) {BBG_WRITE(("Assertion of %s failed at line %d of %s",#Args,__LINE__,__FILE__));} }
#define BBG_CHECKPOINT {BBG_WRITE(("Checkpoint at line %d of %s",__LINE__,__FILE__));}
#define BBG_ENTER(Args) {BBG_WRITE(("%s Entering...",Args));}
#define BBG_LEAVE(Args) {BBG_WRITE(("%s Leaving...",Args));}
#define BBG_EXEC(Args) {Args;}

#else // BUGS_B_GONE

#define BBG_WRITE(Args)
#define BBG_ASSERT(Args)  
#define BBG_CHECKPOINT  
#define BBG_ENTER(Args)
#define BBG_LEAVE(Args)
#define BBG_EXEC(Args)

#endif // BUGS_B_GONE

#endif // BUGS_B_GONE_H
