// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <be/interface/Bitmap.h>
#include <be/interface/Window.h>

#include <be/media/MediaRoster.h>

#include <posix/string.h>

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "EavesdropperMain.h"
#include "IconFactory.h"
#include "LiveNodesView.h"
#include "DormantNodeView.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

DormantNodeView::DormantNodeView(
		InfoWindow* infoWindow,
		dormant_node_info dormantNodeInfo)
	: AbstractNodeView(infoWindow, dormantNodeInfo.name)
{
	// Initialize attributes.
	this->dormantNodeInfo = dormantNodeInfo;

	// Get interesting stuff about the node from the Media Server.
	GetNodeInfo();

	BBG_WRITE(("[DormantNodeView::DormantNodeView] Constructed node %s.",
		dormantNodeInfo.name));
}

// ===========================================================================
// Destructor
// ===========================================================================

DormantNodeView::~DormantNodeView()
{
	// Delete the list of input file formats.
	for (int32 t=0; t<inputFileFormats->CountItems(); ++t)
	{
		delete inputFileFormats->ItemAt(t);
	}
	delete inputFileFormats;
	
	// Delete the list of output file formats.
	for (int32 t=0; t<outputFileFormats->CountItems(); ++t)
	{
		delete outputFileFormats->ItemAt(t);
	}
	delete outputFileFormats;	
}

// ===========================================================================
// Draw
// ===========================================================================

void 
DormantNodeView::Draw(BRect updateRect)
{
	BView::Draw(updateRect);
	DrawNode(dormantNodeInfo.addon, dormantFlavorInfo.kinds);
}

// ===========================================================================
// GetDormantNodeInfo
// ===========================================================================

dormant_node_info
DormantNodeView::GetDormantNodeInfo(void)
{
	return dormantNodeInfo;
}
	
// ===========================================================================
// GetDormantFlavorInfo
// ===========================================================================

dormant_flavor_info
DormantNodeView::GetDormantFlavorInfo(void)
{
	return dormantFlavorInfo;
}

// ===========================================================================
// GetInputFileFormats
// ===========================================================================

BList*
DormantNodeView::GetInputFileFormats(void)
{
	return inputFileFormats;
}

// ===========================================================================
// GetOutputFileFormats
// ===========================================================================

BList*
DormantNodeView::GetOutputFileFormats(void)
{
	return outputFileFormats;
}
	
// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ===========================================================================
// GetNodeInfo
// ===========================================================================

void
DormantNodeView::GetNodeInfo(void)
{
	/*!
	 * [TODO] DormantNodeView::GetNodeInfo - Create just enough memory to hold
	 * the inputs, outputs, file formats, and so on.
	 *
	 * At this time of writing, GetFreeInputsFor() and the other routines
	 * do not return the current number of things you're asking for. If this
	 * still isn't fixed in R4.5 final, then implement Owen's workaround. 
	 */

	BMediaRoster* mediaRoster = BMediaRoster::CurrentRoster();

	inputFileFormats = new BList(5);
	outputFileFormats = new BList(5);

	BBG_WRITE(("[DormantNodeView::GetNodeInfo] Trying to get info for node"
		" %s.", dormantNodeInfo.name));

	// -----------------------------------------------------------------------
	// Get dormant flavor info.
	
	status_t result = mediaRoster->GetDormantFlavorInfoFor(
		dormantNodeInfo,
		&dormantFlavorInfo);

	if (result != B_OK)
	{
		BBG_WRITE(("[DormantNodeView::GetNodeInfo] Could not get dormant"
			" flavor info."));
	}

	// -----------------------------------------------------------------------
	// Get supported file formats for reading.
	
	/*!
	 * [TODO] DormantNodeView::GetNodeInfo - Figure out why I can't get
	 * supported file formats for dormant file interface nodes. Maybe I am
	 * doing something wrong.
	 */
	
	if (dormantFlavorInfo.kinds & B_FILE_INTERFACE)
	{
		const int32 NUMBER_OF_INPUT_FILE_FORMATS_TO_TRY = 50;
		int32 numInputFileFormats = 0;
	
		media_file_format* temp = (media_file_format*)
			malloc(sizeof(media_file_format)
				* NUMBER_OF_INPUT_FILE_FORMATS_TO_TRY);
	
		result = mediaRoster->GetReadFileFormatsFor(
			dormantNodeInfo,
			temp,
			NUMBER_OF_INPUT_FILE_FORMATS_TO_TRY,
			&numInputFileFormats);
	
		if (result != B_OK)
		{
			BBG_WRITE(("[DormantNodeView::GetNodeInfo] Could not get"
				" supported input file formats. (%s)", strerror(result)));
		}
		else
		{
			BBG_WRITE(("[DormantNodeView::GetNodeInfo] Got %ld supported"
				" input file formats.", numInputFileFormats));
	              
			for (int32 t=0; t<numInputFileFormats; ++t)
			{
				inputFileFormats->AddItem(new media_file_format(temp[t]));
			}
		}
		
		free(temp);
	}

	// -----------------------------------------------------------------------
	// Get supported file formats for writing.
	
	if (dormantFlavorInfo.kinds & B_FILE_INTERFACE)
	{
		const int32 NUMBER_OF_OUTPUT_FILE_FORMATS_TO_TRY = 50;
		int32 numOutputFileFormats = 0;
	
		media_file_format* temp = (media_file_format*)
			malloc(sizeof(media_file_format)
				* NUMBER_OF_OUTPUT_FILE_FORMATS_TO_TRY);
	
		result = mediaRoster->GetWriteFileFormatsFor(
			dormantNodeInfo,
			temp,
			NUMBER_OF_OUTPUT_FILE_FORMATS_TO_TRY,
			&numOutputFileFormats);
	
		if (result != B_OK)
		{
			BBG_WRITE(("[DormantNodeView::GetNodeInfo] Could not get"
				" supported output file formats. (%s)", strerror(result)));
		}
		else
		{
			BBG_WRITE(("[DormantNodeView::GetNodeInfo] Got %ld supported"
				" output file formats.", numOutputFileFormats));
	              
			for (int32 t=0; t<numOutputFileFormats; ++t)
			{
				outputFileFormats->AddItem(new media_file_format(temp[t]));
			}
		}
		
		free(temp);
	}
			         	
	// -----------------------------------------------------------------------
}

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
